'==================================================================================================
'
' VBScript Source File 
'
' NAME: ListDistGroups.VBS
' AUTHOR: Bharat Suneja ,
' DATE  : 8/02/2004
' MODIFIED FOR GENERAL USE - 2/13/2006
' MODIFIED FOR FILE OUTPUT - 3/19/2006
'==================================================================================================
' COMMENT: 
'
'
'   Lists all groups in a domain
'   Shows whether group is Dist. Group or Security Group
'   Outputs members of group
'   
'==================================================================================================
'  updated 3/19/2007: Added option to suppress console output, provide a file path for file output, and /HELP option
'==================================================================================================

    
'BANNER INFO
Wscript.Echo "This script lists all groups in the domain"
Wscript.Echo "Format: Group Name, Group Type, Members" 
Wscript.Echo "Report Generated at : " & Now & VbCrLf
strOutput = strOutput & VbCrLf & "Report Generated at : " & Now & VbCrLf

'Pickup Named Arguments
Set colNamedArguments = WScript.Arguments.Named
strOutputFile = colNamedArguments.Item("f")
strNoConOutput = colNamedArguments.Item("s")
strHelp = colNamedArguments.Item("help")

'Detect help and write help text to console
WriteHelp

'Get RootDSE
Set objRootDSE = GetObject("LDAP://rootDSE")
strDomain = objRootDSE.Get("defaultNamingContext")
strADPath = "LDAP://" & strDomain
Set objDomain = GetObject(strADPath)
wscript.echo "Domain: " & objDomain.distinguishedName




'Setup ADODB connection
Set objConnection = CreateObject("ADODB.Connection")
objConnection.Open "Provider=ADsDSOObject;"

Set objCommand = CreateObject("ADODB.Command")
objCommand.ActiveConnection = objConnection

    'Execute search command to look for user
    objCommand.CommandText = _
      "<" & strADPath & ">" & ";(&(objectClass=group)(mail=*))" & ";distinguishedName,member;subtree"
    Set objRecordSet = objCommand.Execute

Wscript.Echo "DISTRIBUTION GROUPS FOUND: " & objRecordSet.RecordCount & VbCrlf
Wscript.Echo "--------------------------------------------------------------------------------------------" & VbCrlf & VbCrlf
       I = 0

       While Not objRecordSet.EOF 'Iterate through the search results
            I = I + 1
            strGroupDN = objRecordSet.Fields("distinguishedName")
            set objGroup= GetObject("LDAP://"& strGroupDN & "")
            
            strFirstLine = ""
            strFirstLine = strFirstLIne & I & "#" & objGroup.sAMAccountName & " # "
            If objGroup.GroupType = -2147483646 Then
              strFirstLine = strFirstLIne & "Security Group" & "#"
              Else 
              strFirstLine = strFirstLIne & "Distribution Group" & "#"
              End If
              
              strDescription = objGroup.description
              strFirstLine = strFirstLine & strDescription & "#"
              
            'Get manager's name
            If objGroup.managedBy = "" Then
               'Wscript.Echo "This group does not have a Manager."
               strFirstLine = strFirstLine & "No Manager" & "#"
               
               Else 
                  strManagerDN = objGroup.managedBy
                  set objManager = GetObject("LDAP://"& strManagerDN & "")
                  strManagerName = objManager.displayName
                  strFirstLine = strFirstLine & strManagerName & "#"
               End If
            
                        
            'strWhenCreated = objRecordSet.Fields("whenCreated")             _
                     'Shows when the account was created, disable if not required
            

                  'Group operation: Gets group memberships, checks if Distribution Group, deletes if Distribution Group
                  If IsNull (objRecordSet.Fields("member")) Then    'Checks if the user has already been removed from all groups
                strFirstLine = strFirstLIne  & "---NONE---"             
                Else
                  'strFirstLine = strFirstLIne &  VbCrlf & "Members: "

                  
            
                  For each varUser in objRecordSet.Fields("member").Value
                      
                      Set objUser = GetObject("LDAP://" & varUser)
                      
                      'Wscript.Echo "Group Type= " &  objUser.Grouptype
                      
                          strFirstLine = strFirstLIne &  objUser.displayName & "," 
                          
                      
                  Next
                  End If
                  
       

         
            

            
            'strFirstLine = strFirstLIne &  "#" 
           'Wscript.Echo "-----------------------" & VbCrlf
            Wscript.Echo strFirstLine
            strOutput = strOutput & VbCrLf & strFirstLine
       objRecordSet.MoveNext
       Wend
       
        WriteCon
              WriteFile
            
       
       
       
         '=======================================================
              '  FUNCTIONS
              '=======================================================
              
              Sub WriteFile
              'Open new text file, write info, close file
                If IsEmpty(strOutputFile) Then
                   WScript.Echo "No file output"
                 Else
                     Set objFSO = CreateObject("Scripting.FileSystemObject")
                     Set objFile = objFSO.CreateTextFile(strOutputFile)
                     'Write Stuff
                    objFile.WriteLine strOutput
                    'Close text file
                    objFile.Close
                    End If
                  End Sub
            
                  '---------------------------------
      'SUB Write Console
       Sub WriteCon
          If Not colNamedArguments.Exists("s") Then
          WScript.echo strOutput
          Else
          WScript.Echo "No console output"
          End If
       End Sub
       
            '------------------
            
    Sub WriteHelp
       If colNamedArguments.Exists("help") Then
          strHelpText = "USAGE: ListDistGroups_audit-dev.vbs" & VbCrLf & "OPTIONAL Arguments: /s:y   - no console output" & VbCrLf & _
          "/f:Output_File_Name.txt - writes output to txt file in the argument" & VbCrLf & "/help - help text" & VbCrLf & _
          " ListDistGroups_audit-dev.vbs /f:c:\DistributionGroups.txt /s:y - produces " & VbCrLf & "file DistributionGroups.txt " & " and no console output"
       WScript.Echo strHelpText
       WScript.Quit
       End If
       End Sub

            


            




